﻿using System;
using System.Web.UI;
using Telerik.Web.UI;
using System.Collections.Generic;

public partial class _Default : Page
{
    private const int AppointmentsLimit = 1;
    private const int LookaheadDays = 30;
    private object RecurrenceParentID;
    private bool SkipLimitCheck = false;

    protected bool ExceedsLimit(Appointment apt)
    {
        bool exceedsLimit = false;
        int appointmentsCount = 0;

        // Prepare a list of all appointment occurrences, including the first
        List<DateTime> occurrences = new List<DateTime>();
        if (string.IsNullOrEmpty(apt.RecurrenceRule))
        {
            occurrences.Add(apt.Start);
        }
        else
        {
            RecurrenceRule rrule;
            if (RecurrenceRule.TryParse(apt.RecurrenceRule, out rrule))
            {
                rrule.SetEffectiveRange(apt.Start, apt.End.AddDays(LookaheadDays));
                occurrences.AddRange(rrule.Occurrences);
            }
        }

        foreach (DateTime occStart in occurrences)
        {
            DateTime occEnd = occStart.Add(apt.Duration);
            foreach (Appointment existingApt in RadScheduler1.Appointments)
            {
                bool isSameAppointment = existingApt.ID.ToString().Equals(apt.Attributes["ID"]);

                bool isChildOccurrence;
                if (existingApt.RecurrenceState == RecurrenceState.Exception)
                {
                    isChildOccurrence = existingApt.RecurrenceState == RecurrenceState.Exception && existingApt.RecurrenceParentID.ToString().Equals(apt.Attributes["ID"]);
                }
                else
                {
                    isChildOccurrence = existingApt.RecurrenceState == RecurrenceState.Occurrence && existingApt.RecurrenceParentID.ToString().Equals(apt.Attributes["ID"]);
                }
                bool isOccurrenceOfSameParent = RecurrenceParentID != null && existingApt.RecurrenceState == RecurrenceState.Occurrence && existingApt.RecurrenceParentID.ToString().Equals(apt.Attributes["ID"]);

                if (existingApt.Visible && AppointmentOverlaps(existingApt, occStart, occEnd) &&
                    !isSameAppointment && !isChildOccurrence && !isOccurrenceOfSameParent)
                {
                    appointmentsCount++;
                }

                if (appointmentsCount >= AppointmentsLimit)
                {
                    exceedsLimit = true;
                    break;
                }
            }
        }

        return exceedsLimit;
    }

    protected void RadScheduler1_AppointmentInsert(object sender, Telerik.Web.UI.SchedulerCancelEventArgs e)
    {
        if (ExceedsLimit(e.Appointment))
        {
            e.Cancel = true;
            ShowLimitMessage();
        }
    }
    string resetRecurrence = null;
    protected void RadScheduler1_AppointmentUpdate(object sender, Telerik.Web.UI.AppointmentUpdateEventArgs e)
    {
        if (ExceedsLimit(e.ModifiedAppointment))
        {
            if (e.Appointment.RecurrenceState == RecurrenceState.Master)
            {
                resetRecurrence = e.Appointment.ID.ToString();
            }
            else
            {
                resetRecurrence = null;
            }
            e.Cancel = true;
            ShowLimitMessage();
        }
    }

    protected void RadScheduler1_RecurrenceExceptionCreated(object sender, RecurrenceExceptionCreatedEventArgs e)
    {
        // Don't check for limits while creating recurrence exception as it replaces existing occurrence
        RecurrenceParentID = e.Appointment.ID;

        if (ExceedsLimit(e.ExceptionAppointment))
        {
            e.Cancel = true;
            ShowLimitMessage();
        }
    }

    private void ShowLimitMessage()
    {
        Label1.Text = "Another appointment exists in this time slot.";

        ScriptManager.RegisterClientScriptBlock(this, GetType(), "LabelUpdated",
            "$telerik.$('.lblError').show().animate({ opacity: 0.9 }, 2000).fadeOut('slow');", true);
    }

    private bool AppointmentOverlaps(Appointment apt, DateTime rangeStart, DateTime rangeEnd)
    {
        bool overlaps = apt.Start < rangeEnd && apt.End > rangeStart;

        if (apt.Duration == TimeSpan.Zero)
        {
            return overlaps || apt.Start == rangeStart;
        }

        return overlaps;
    }

    private bool RoomsOverlap(Appointment aptOne, Appointment aptTwo)
    {
        foreach (Resource room in aptOne.Resources.GetResourcesByType("Room"))
        {
            if (aptTwo.Resources.GetResource(room.Type, room.Key) != null)
            {
                return true;
            }
        }

        return false;
    }

    protected void RadScheduler1_AppointmentCancelingEdit(object sender, AppointmentCancelingEditEventArgs e)
    {
        Label1.Text = "";
    }

    protected void Page_Load(object sender, EventArgs e)
    {

    }

    protected void RadScheduler1_DataBound(object sender, EventArgs e)
    {
        RadScheduler1.ResourceTypes.FindByName("Cook").AllowMultipleValues = true;
        RadScheduler1.ResourceTypes.FindByName("Server").AllowMultipleValues = true;
    }
    protected void RadScheduler1_AppointmentCreated(object sender, AppointmentCreatedEventArgs e)
    {
        e.Appointment.Attributes["ID"] = e.Appointment.ID.ToString();
    }
    protected void RadScheduler1_AppointmentDelete(object sender, AppointmentDeleteEventArgs e)
    {
        if (e.Appointment.RecurrenceState == RecurrenceState.Exception)
        {
            if (resetRecurrence == e.Appointment.RecurrenceParentID.ToString())
            {
                e.Cancel = true;
            }

        }

    }
}
