﻿Imports System
Imports System.Web.UI
Imports Telerik.Web.UI
Imports System.Collections.Generic

Partial Class DefaultVB
    Inherits System.Web.UI.Page

    Private Const AppointmentsLimit As Integer = 1
    Private Const LookaheadDays As Integer = 30
    Private RecurrenceParentID As Object
    Private SkipLimitCheck As Boolean = False

    Protected Function ExceedsLimit(ByVal apt As Appointment) As Boolean
        Dim eLimit As Boolean = False
        Dim appointmentsCount As Integer = 0

        ' Prepare a list of all appointment occurrences, including the first
        Dim occurrences As New List(Of DateTime)()
        If String.IsNullOrEmpty(apt.RecurrenceRule) Then
            occurrences.Add(apt.Start)
        Else
            Dim rrule As RecurrenceRule
            If RecurrenceRule.TryParse(apt.RecurrenceRule, rrule) Then
                rrule.SetEffectiveRange(apt.Start, apt.[End].AddDays(LookaheadDays))
                occurrences.AddRange(rrule.Occurrences)
            End If
        End If

        For Each occStart As DateTime In occurrences
            Dim occEnd As DateTime = occStart.Add(apt.Duration)
            For Each existingApt As Appointment In RadScheduler1.Appointments
                Dim isSameAppointment As Boolean = existingApt.ID.ToString().Equals(apt.Attributes("ID"))

                Dim isChildOccurrence As Boolean
                If existingApt.RecurrenceState = RecurrenceState.Exception Then
                    isChildOccurrence = existingApt.RecurrenceState = RecurrenceState.Exception AndAlso existingApt.RecurrenceParentID.ToString().Equals(apt.Attributes("ID"))
                Else
                    isChildOccurrence = existingApt.RecurrenceState = RecurrenceState.Occurrence AndAlso existingApt.RecurrenceParentID.ToString().Equals(apt.Attributes("ID"))
                End If
                Dim isOccurrenceOfSameParent As Boolean = RecurrenceParentID IsNot Nothing AndAlso existingApt.RecurrenceState = RecurrenceState.Occurrence AndAlso existingApt.RecurrenceParentID.ToString().Equals(apt.Attributes("ID"))

                If existingApt.Visible AndAlso AppointmentOverlaps(existingApt, occStart, occEnd) AndAlso RoomsOverlap(apt, existingApt) AndAlso Not isSameAppointment AndAlso Not isChildOccurrence AndAlso Not isOccurrenceOfSameParent Then
                    appointmentsCount += 1
                End If

                If appointmentsCount >= AppointmentsLimit Then
                    eLimit = True
                    Exit For
                End If
            Next
        Next

        Return eLimit
    End Function

    Protected Sub RadScheduler1_AppointmentInsert(ByVal sender As Object, ByVal e As Telerik.Web.UI.SchedulerCancelEventArgs)
        If ExceedsLimit(e.Appointment) Then
            e.Cancel = True
            ShowLimitMessage()
        End If
    End Sub
    Private resetRecurrence As String = Nothing
    Protected Sub RadScheduler1_AppointmentUpdate(ByVal sender As Object, ByVal e As Telerik.Web.UI.AppointmentUpdateEventArgs)
        If ExceedsLimit(e.ModifiedAppointment) Then
            If e.Appointment.RecurrenceState = RecurrenceState.Master Then
                resetRecurrence = e.Appointment.ID.ToString()
            Else
                resetRecurrence = Nothing
            End If
            e.Cancel = True
            ShowLimitMessage()
        End If
    End Sub

    Protected Sub RadScheduler1_RecurrenceExceptionCreated(ByVal sender As Object, ByVal e As RecurrenceExceptionCreatedEventArgs)
        ' Don't check for limits while creating recurrence exception as it replaces existing occurrence
        RecurrenceParentID = e.Appointment.ID

        If ExceedsLimit(e.ExceptionAppointment) Then
            e.Cancel = True
            ShowLimitMessage()
        End If
    End Sub

    Private Sub ShowLimitMessage()
        Label1.Text = "Another appointment exists in this time slot."

        ScriptManager.RegisterClientScriptBlock(Me, [GetType](), "LabelUpdated", "$telerik.$('.lblError').show().animate({ opacity: 0.9 }, 2000).fadeOut('slow');", True)
    End Sub

    Private Function AppointmentOverlaps(ByVal apt As Appointment, ByVal rangeStart As DateTime, ByVal rangeEnd As DateTime) As Boolean
        Dim overlaps As Boolean = apt.Start < rangeEnd AndAlso apt.[End] > rangeStart

        If apt.Duration = TimeSpan.Zero Then
            Return overlaps OrElse apt.Start = rangeStart
        End If

        Return overlaps
    End Function

    Private Function RoomsOverlap(ByVal aptOne As Appointment, ByVal aptTwo As Appointment) As Boolean
        For Each room As Resource In aptOne.Resources.GetResourcesByType("Room")
            If aptTwo.Resources.GetResource(room.Type, room.Key) IsNot Nothing Then
                Return True
            End If
        Next

        Return False
    End Function

    Protected Sub RadScheduler1_AppointmentCancelingEdit(ByVal sender As Object, ByVal e As AppointmentCancelingEditEventArgs)
        Label1.Text = ""
    End Sub

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)

    End Sub

    Protected Sub RadScheduler1_DataBound(ByVal sender As Object, ByVal e As EventArgs)
        RadScheduler1.ResourceTypes.FindByName("Cook").AllowMultipleValues = True
        RadScheduler1.ResourceTypes.FindByName("Server").AllowMultipleValues = True
    End Sub
    Protected Sub RadScheduler1_AppointmentCreated(ByVal sender As Object, ByVal e As AppointmentCreatedEventArgs)
        e.Appointment.Attributes("ID") = e.Appointment.ID.ToString()
    End Sub
    Protected Sub RadScheduler1_AppointmentDelete(ByVal sender As Object, ByVal e As AppointmentDeleteEventArgs)
        If e.Appointment.RecurrenceState = RecurrenceState.Exception Then
            If resetRecurrence = e.Appointment.RecurrenceParentID.ToString() Then
                e.Cancel = True

            End If
        End If

    End Sub

End Class
